# -*- coding: utf-8 -*-

"""

Module for downloading Sentinel-2 images using EODAG
https://www.github.com/CS-SI/EODAG

"""
import logging
import shapefile
import os
from pathlib import Path
from eodag import EODataAccessGateway
from .config import SHARED_DATA, Config


logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)

ROOT = Path(os.path.realpath(__file__)).parent.parent

class S2cEodag:

    def __init__(self, name: str):
        self.name = name
        self.products = None
        eodag_centroids_shp = SHARED_DATA.get("eodag_centroids_shp")
        
        with shapefile.Reader(eodag_centroids_shp) as shp:
            shaperecs = shp.shapeRecords()
        
        locations_yaml_content = """
        shapefiles:
          - name: s2_tile_centroid
            path: {}
            attr: tile_id
        """.format(os.path.abspath(eodag_centroids_shp))
        
        eodag_custom_location_cfg = os.path.abspath(Config()._CONFIG_DIR / "eodag_custom_locations.yml")

        with open(eodag_custom_location_cfg, "w") as f_yml:
            f_yml.write(locations_yaml_content.strip())
        
        self.dag = EODataAccessGateway(locations_conf_path = eodag_custom_location_cfg)
        self.dag.set_preferred_provider("peps")
        
        logger.info(self.dag.available_providers("S2_MSI_L1C"))

        targeted_tile = [
            sr
            for sr in shaperecs
            if sr.record["tile_id"] == name
        ][0]
        
    
    def search(
        self,
        productType: str = "L1C",
        start: str = "2015-01-01",
        end: str = "9999-01-01",
    
    ):
        if productType == "L1C":
            self.productType = "S2_MSI_L1C"
            outputs_prefix = Path(Config().get("l1c_path")) / self.name
        elif productType == "L2A":
            self.productType = "S2_MSI_L2A"
            outputs_prefix = Path(Config().get("l2a_path")) / self.name

        default_search_criteria = dict(
            productType = self.productType,
            start = start,
            end = end
        )
        
        self.products = self.dag.search_all(
            locations=dict(s2_tile_centroid=self.name),
            **default_search_criteria
        )
        
        #### Vériffier ici si le produit existe déjà en local
        # ~ logger.info(outputs_prefix)
        
        for p in self.products:
            

            # ~ logger.info("location: {}".format(p.location))
            if (outputs_prefix / (p.properties["title"] + ".SAFE")).exists():
                p.location = "file://" + str(outputs_prefix / (p.properties["title"] + ".SAFE"))
                logger.info("Product {} locally present ".format(p.properties["title"]))
            else:
                logger.info("Product {} locally absent ".format(p.properties["title"]))
            # ~ logger.info("location: {}".format(p.location))
            
        # ~ logger.info("{} products were found given the above search criteria".format(len(self.products)))
        # ~ logger.info(self.products)
        # ~ logger.info([p.properties["storageStatus"] for p in self.products])
        
        # ~ logger.info("{} products were found given the above search criteria".format(len(self.products)))
        # ~ logger.info("{} products were found given the above search criteria, online".format(len(self.products.filter_online())))

    def download(
        self,
        product_id, 
        outputs_prefix: str = None,
        extract: bool = True,
        delete_archive: bool = True,
    ):
        ######### faudrait ici récupérer le type de produit L1C/L2A du nom du product_id et adapter le dossier en conséquence
        if not outputs_prefix:
            if self.productType == "S2_MSI_L1C":
                root_path = "l1c_path"
            elif self.productType == "S2_MSI_L2A":
                root_path = "l2a_path"
            outputs_prefix = str(Path(Config().get(root_path)) / self.name)
        
        ########## faudrait aussi mettre une condition sur le online
        ########## et si offline supprimer les retry
        downloaded_path = self.dag.download(
            product_id, 
            outputs_prefix = outputs_prefix, 
            extract = extract, 
            delete_archive = delete_archive,
        )

        if Path(downloaded_path).stem == Path(downloaded_path).parent.stem:
            upper_location = Path(downloaded_path).parent.parent / Path(downloaded_path).name
            Path(downloaded_path).replace(upper_location)
            product_id.location = "file://" + str(upper_location)
            Path(downloaded_path).parent.rmdir()
            logger.info("Moving up SAVE file")


